'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.compareSnapshot = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _jestMatchers = require('jest-matchers');

var _jestMatchers2 = _interopRequireDefault(_jestMatchers);

var _jsWriter = require('js-writer');

var _jsWriter2 = _interopRequireDefault(_jsWriter);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _unexpectedHtmllikeRawAdapter = require('unexpected-htmllike-raw-adapter');

var _unexpectedHtmllikeRawAdapter2 = _interopRequireDefault(_unexpectedHtmllikeRawAdapter);

var _snapshotLoader = require('./snapshotLoader');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var UnexpectedSnapshotState = function () {
  function UnexpectedSnapshotState(snapshotState) {
    _classCallCheck(this, UnexpectedSnapshotState);

    var files = {};
    this._files = files;
  }

  _createClass(UnexpectedSnapshotState, [{
    key: 'getSnapshot',
    value: function getSnapshot(testPath, testName) {
      var snapshot = this._files[testPath];
      if (!snapshot) {
        var snapshotPath = getSnapshotPath(testPath);
        var content = (0, _snapshotLoader.loadSnapshot)(snapshotPath);

        snapshot = this._files[testPath] = {
          testCounter: {},
          uncheckedKeys: content && new Set(Object.keys(content)) || new Set(),
          allTests: content || {}
        };
      }
      var count = (snapshot.testCounter[testName] || 0) + 1;
      snapshot.testCounter[testName] = count;

      var keyName = testName + ' ' + count;
      console.log('removing', keyName);
      snapshot.uncheckedKeys.delete(keyName);
      return snapshot.allTests[keyName] || null;
    }
  }, {
    key: 'saveSnapshot',
    value: function saveSnapshot(testPath, testName, tree) {

      var snapshotPath = getSnapshotPath(testPath);
      var snapshot = this._files[testPath];
      var count = snapshot.testCounter[testName] || 1;

      snapshot.allTests[testName + ' ' + count] = tree;
      var dir = _path2.default.dirname(snapshotPath);
      var exists = void 0;
      try {
        exists = _fs2.default.statSync(dir).isDirectory();
      } catch (e) {
        exists = false;
      }
      if (!exists) {
        _mkdirp2.default.sync(dir);
      }
      var fileContent = Object.keys(snapshot.allTests).map(function (test) {
        return 'exports[`' + test + '`] = ' + (0, _jsWriter2.default)(snapshot.allTests[test]) + ';';
      }).join('\n\n');
      _fs2.default.writeFileSync(snapshotPath, fileContent);
    }
  }]);

  return UnexpectedSnapshotState;
}();

function getSnapshotPath(testPath) {
  var testPathParsed = _path2.default.parse(testPath);
  testPathParsed.dir = _path2.default.join(testPathParsed.dir, '__snapshots__');
  testPathParsed.base = testPathParsed.name + '.unexpected-snap';

  return _path2.default.format(testPathParsed);
}

var rawAdapter = new _unexpectedHtmllikeRawAdapter2.default({ convertToString: true, concatTextContent: true });

function compareSnapshot(expect, flags, subjectAdapter, subjectRenderer, subjectOutput) {

  var state = _jestMatchers2.default.getState();

  if (!state.unexpectedSnapshot) {
    state.unexpectedSnapshot = new UnexpectedSnapshotState(state.snapshotState);
  }

  var snapshot = state.unexpectedSnapshot.getSnapshot(state.testPath, state.currentTestName);
  if (snapshot === null) {
    // Write and save
    state.unexpectedSnapshot.saveSnapshot(state.testPath, state.currentTestName, rawAdapter.serialize(subjectAdapter, subjectOutput));
    state.snapshotState.added++;
  } else {
    expect.withError(function () {
      expect(subjectRenderer, 'to have rendered', rawAdapter.deserialize(snapshot));
      state.snapshotState.matched = (state.snapshotState.matched || 0) + 1;
    }, function (err) {
      if (state.snapshotState.update === true) {
        state.snapshotState.updated++;
        state.unexpectedSnapshot.saveSnapshot(state.testPath, state.currentTestName, rawAdapter.serialize(subjectAdapter, subjectOutput));
      } else {
        state.snapshotState.unmatched++;
        expect.fail(err);
      }
    });
  }
}

function hookRemoveUncheckedKeys() {
  var state = _jestMatchers2.default.getState();
  var snapshotState = state && state.snapshotState;
  console.log('hooking into snapshotState', snapshotState);
  if (snapshotState) {
    (function () {
      var originalRemoveUncheckedKeys = snapshotState.removeUncheckedKeys;

      snapshotState.removeUncheckedKeys = function () {
        var state = _jestMatchers2.default.getState();
        console.log('running unexpected removeUncheckedKeys', state.testPath);
        var snapshot = state.unexpectedSnapshot && state.unexpectedSnapshot._files[state.testPath];
        if (snapshot && snapshot.uncheckedKeys.size) {
          snapshot.uncheckedKeys.forEach(function (key) {
            delete snapshot.allTests[key];
          });
        }

        if (!snapshot || Object.keys(snapshot.allTests).length === 0) {
          var snapshotPath = getSnapshotPath(state.testPath);
          try {
            if (_fs2.default.statSync(snapshotPath).isFile()) {
              _fs2.default.unlinkSync(getSnapshotPath(state.testPath));
            }
          } catch (e) {
            // We're ignoring file-not-found exceptions, and errors deleting
          }

          if (state.unexpectedSnapshot) delete state.unexpectedSnapshot._files[state.testPath];
        }
        console.log('calling original remove unchecked');
        originalRemoveUncheckedKeys.call(snapshotState);
      };
    })();
  }
}

hookRemoveUncheckedKeys();

exports.compareSnapshot = compareSnapshot;